from django.contrib.auth import get_user_model
from django.urls import reverse

from rest_framework.test import APIClient
from rest_framework import status

from jaseci.utils.utils import TestCaseHelper
from django.test import TestCase


# Consts for url
CREATE_USER_URL = reverse('user_api:create')
TOKEN_URL = reverse('user_api:token')
MANAGE_URL = reverse('user_api:manage')
LOGOUT_EVERYONE_URL = reverse('user_api:logout_everyone')
PASSWORD_RESET_URL = reverse(
    'user_api:password_reset:reset-password-request')
# Alias for create user
create_user = get_user_model().objects.create_user
create_superuser = get_user_model().objects.create_superuser
get_user = get_user_model().objects.get


class user_api_tests_public(TestCaseHelper, TestCase):
    """Tests for user API (public)"""

    def setUp(self):
        super().setUp()
        self.client = APIClient()

    def tearDown(self):
        super().tearDown()

    def test_create_valid_user_success(self):
        """Test creating user with valid payload is successful"""
        payload = {
            'email': 'jscitest_test2@jaseci.com',
            'password': 'testpass',
            'name': 'name'
        }
        res = self.client.post(CREATE_USER_URL, payload)

        self.assertEqual(res.status_code, status.HTTP_201_CREATED)
        user = get_user_model().objects.get(**res.data)
        self.assertTrue(
            user.check_password(payload['password'])
        )
        self.assertNotIn('password', res.data)
        user.delete()

    def test_first_user_is_super(self):
        """Test first user created its superuser"""
        payload = {
            'email': 'jscitest_test2@jaseci.com',
            'password': 'testpass',
            'name': 'name'
        }
        res = self.client.post(CREATE_USER_URL, payload)

        self.assertEqual(res.status_code, status.HTTP_201_CREATED)
        user = get_user_model().objects.get(**res.data)
        self.assertTrue(user.is_admin)
        payload = {
            'email': 'jscitest_test4@jaseci.com',
            'password': 'testpass',
            'name': 'name'
        }
        res = self.client.post(CREATE_USER_URL, payload)

        self.assertEqual(res.status_code, status.HTTP_201_CREATED)
        user2 = get_user_model().objects.get(**res.data)
        self.assertFalse(user2.is_admin)
        user.delete()
        user2.delete()

    def test_create_activated_user_success(self):
        """Test creating user with valid payload is successful"""
        payload = {
            'email': 'jscitest_test2@jaseci.com',
            'password': 'testpass',
            'name': 'name',
            'is_activated': True
        }
        res = self.client.post(CREATE_USER_URL, payload)

        self.assertEqual(res.status_code, status.HTTP_201_CREATED)
        user = get_user_model().objects.get(**res.data)
        self.assertTrue(
            user.check_password(payload['password'])
        )
        self.assertTrue(user.is_activated)
        self.assertNotIn('password', res.data)
        user.delete()

    def test_user_exists(self):
        """Test creating a user that already exists fails"""
        payload = {
            'email': 'jscitest_test2@jaseci.com',
            'password': 'testpass'
        }
        create_user(**payload)
        res = self.client.post(CREATE_USER_URL, payload)

        self.assertEqual(res.status_code, status.HTTP_400_BAD_REQUEST)
        get_user(email=payload['email']).delete()

    def test_password_too_short(self):
        """Test that passwords less than 8 characters fail"""
        payload = {
            'email': 'jscitest_test2@jaseci.com',
            'password': 'pw'
        }
        res = self.client.post(CREATE_USER_URL, payload)

        self.assertEqual(res.status_code, status.HTTP_400_BAD_REQUEST)
        user_exists = get_user_model().objects.filter(
            email=payload['email']
        ).exists()
        self.assertFalse(user_exists)

    def test_logout_everyone(self):
        """Test admin level logout everyone"""
        payload = {'email': 'jscitest_test@jaseci.com', 'password': 'testpass'}
        user = create_user(**payload)
        user.is_activated = True
        user.save()
        payload2 = {'email': 'super@jaseci.com', 'password': 'testpass'}
        superuser = create_superuser(**payload2)
        superuser.save()

        res = self.client.post(TOKEN_URL, payload)
        token = res.data['token']
        res = self.client.post(TOKEN_URL, payload2)
        stoken = res.data['token']
        res = self.client.get(MANAGE_URL)
        self.assertEqual(res.status_code, status.HTTP_401_UNAUTHORIZED)
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + token)
        res = self.client.get(MANAGE_URL)
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        res = self.client.get(MANAGE_URL)
        self.assertEqual(res.status_code, status.HTTP_200_OK)

        self.client.credentials(HTTP_AUTHORIZATION='Token ' + stoken)
        self.client.post(LOGOUT_EVERYONE_URL, {})
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + token)
        res = self.client.get(MANAGE_URL)
        self.assertEqual(res.status_code, status.HTTP_401_UNAUTHORIZED)
        get_user(email=payload['email']).delete()
        get_user(email=payload2['email']).delete()

    def test_logout_everyone_requires_admin(self):
        """Test admin level logout everyone"""
        payload2 = {'email': 'super@jaseci.com', 'password': 'testpass'}
        superuser = create_superuser(**payload2)
        superuser.save()
        payload = {'email': 'jscitest_test@jaseci.com', 'password': 'testpass'}
        user = create_user(**payload)
        user.is_activated = True
        user.save()

        res = self.client.post(TOKEN_URL, payload)
        token = res.data['token']
        res = self.client.post(TOKEN_URL, payload2)
        # stoken = res.data['token']
        res = self.client.get(MANAGE_URL)
        self.assertEqual(res.status_code, status.HTTP_401_UNAUTHORIZED)
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + token)
        res = self.client.get(MANAGE_URL)
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        res = self.client.get(MANAGE_URL)
        self.assertEqual(res.status_code, status.HTTP_200_OK)

        self.client.credentials(HTTP_AUTHORIZATION='Token ' + token)
        res = self.client.post(LOGOUT_EVERYONE_URL, {})
        self.assertEqual(res.status_code, status.HTTP_403_FORBIDDEN)
        self.client.credentials(HTTP_AUTHORIZATION='Token ' + token)
        res = self.client.get(MANAGE_URL)
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        get_user(email=payload['email']).delete()
        get_user(email=payload2['email']).delete()

    def test_create_token_for_user(self):
        """Test that a token is created for the user"""
        payload = {'email': 'jscitest_test@jaseci.com', 'password': 'testpass'}
        user = create_user(**payload)
        user.is_activated = True
        user.save()
        res = self.client.post(TOKEN_URL, payload)

        # self.assertIsNotNone(get_user(email=payload['email']).hook)

        self.assertIn('token', res.data)
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        get_user(email=payload['email']).delete()

    def test_create_token_invalid_credentials(self):
        """Test that token is not created if invalid credentials are given"""
        create_user(email='jscitest_test@jaseci.com', password='testpass')
        payload = {'email': 'jscitest_test@jaseci.com', 'password': 'wrong'}
        res = self.client.post(TOKEN_URL, payload)

        self.assertNotIn('token', res.data)
        self.assertEqual(res.status_code, status.HTTP_400_BAD_REQUEST)
        get_user(email=payload['email']).delete()

    def test_create_token_no_user(self):
        """Test that token is not created if user doens't exist"""
        payload = {'email': 'jscitest_test@jaseci.com', 'password': 'testpass'}
        res = self.client.post(TOKEN_URL, payload)

        self.assertNotIn('token', res.data)
        self.assertEqual(res.status_code, status.HTTP_400_BAD_REQUEST)

    def test_create_token_missing_field(self):
        """Test that email and password are required"""
        res = self.client.post(TOKEN_URL, {'email': 'one', 'password': ''})
        self.assertNotIn('token', res.data)
        self.assertEqual(res.status_code, status.HTTP_400_BAD_REQUEST)

    def test_retrieve_user_unauthorized(self):
        """Test that authentication required for users"""
        res = self.client.get(MANAGE_URL)

        self.assertEqual(res.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_password_reset_endpoint(self):
        """Test that password resent endpoint succeeds for the user"""
        payload = {'email': 'jscitest_test@jaseci.com', 'password': 'testpass'}
        user = create_user(**payload)
        user.is_activated = True
        user.save()

        email_pl = {'email': 'jscitest_test@jaseci.com'}
        from jaseci_serv.base.mail import email_config_defaults
        email_config_defaults['EMAIL_BACKEND'] = \
            'django.core.mail.backends.locmem.EmailBackend'

        res = self.client.post(PASSWORD_RESET_URL, email_pl)

        self.assertEqual(res.status_code, status.HTTP_200_OK)
        get_user(email=payload['email']).delete()

    def test_password_reset_endpoint_invalid_email(self):
        """Test that password resent endpoint succeeds for the user"""
        payload = {'email': 'jscitest_test@jaseci.com', 'password': 'testpass'}
        user = create_user(**payload)
        user.is_activated = True
        user.save()

        email_pl = {'email': 'jjscitest_test@jaseci.com'}
        from jaseci_serv.base.mail import email_config_defaults
        email_config_defaults['EMAIL_BACKEND'] = \
            'django.core.mail.backends.locmem.EmailBackend'

        res = self.client.post(PASSWORD_RESET_URL, email_pl)

        self.assertEqual(res.status_code, status.HTTP_400_BAD_REQUEST)
        get_user(email=payload['email']).delete()


class user_api_tests_private(TestCaseHelper, TestCase):
    """Test API requests that are authenticated"""

    def setUp(self):
        super().setUp()
        self.user = create_user(
            email='jscitest_test@jaseci.com',
            password='testpass',
            name='some dude',
        )
        self.client = APIClient()
        self.client.force_authenticate(user=self.user)

    def tearDown(self):
        self.user.delete()
        super().tearDown()

    def test_retrieve_user_success(self):
        """Test retrieving profile for logged in user"""
        res = self.client.get(MANAGE_URL)

        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertEqual(res.data, {
            'id': self.user.id,
            'name': self.user.name,
            'email': self.user.email,
            'is_activated': self.user.is_activated,
        })

    def test_post_me_not_allowed(self):
        """Test that POST is not allowed to update user"""
        res = self.client.post(MANAGE_URL, {})

        self.assertEqual(res.status_code, status.HTTP_405_METHOD_NOT_ALLOWED)

    def test_update_user_profile(self):
        """Test updating the user profile for authenticated user"""
        payload = {'name': 'new name', 'password': 'newpassword123'}

        res = self.client.patch(MANAGE_URL, payload)

        self.user.refresh_from_db()
        self.assertEqual(self.user.name, payload['name'])
        self.assertTrue(self.user.check_password(payload['password']))
        self.assertEqual(res.status_code, status.HTTP_200_OK)

    def test_update_user_profile_via_put(self):
        """Test updating the user profile for authenticated user"""
        payload = {'name': 'new name', 'password': 'newpassword123',
                   "email": "mars.ninja@gmail.com", "is_activated": True}

        res = self.client.put(MANAGE_URL, payload)

        self.user.refresh_from_db()
        self.assertEqual(self.user.name, payload['name'])
        self.assertTrue(self.user.check_password(payload['password']))
        self.assertEqual(res.status_code, status.HTTP_200_OK)



from django.urls import path
from .views import AbstractJacAPIView
from .views import AbstractAdminJacAPIView, AbstractPublicJacAPIView
from jaseci.element.element import element
from jaseci_serv.base.models import super_master
from jaseci.utils.utils import copy_func
from inspect import signature
import uuid
import json


def rest_api_auto_doc(endpoint: str, fsig: signature):
    """
    Automatically return string for REST Interface documentation
    Parameters include endpoint uri and signature
    """
    doc = f"**REST Endpoint**: {endpoint}\n"
    params = []
    json_samp = {}

    for i in fsig.parameters.keys():
        if (i == 'self'):
            continue
        p_name = i
        p_type = fsig.parameters[i].annotation
        if(issubclass(p_type, element)):
            params.append(
                f'> {p_name}: UUID pointing to {p_type.__name__} object\n')
            json_samp[p_name] = uuid.uuid4().urn
        else:
            params.append(f'> {p_name}: type {p_type.__name__}\n')
            json_samp[p_name] = p_type()
            if (p_type == str):
                json_samp[p_name] = 'some string'
    if (params):
        doc += "\n**Parameters**:\n"
        for i in params:
            doc += i
        doc += "\n**JSON Example**\n"
        doc += f"```javascript\n{json.dumps(json_samp, indent=4)}\n```\n"
    return doc


generated_urls = []


def generate_apis(api_list, view_cls, dir_head):
    """
    Auto generates Django APIs based on core interface
    """
    for i in api_list:
        fname = '_'.join(i['groups'])
        apidocstr = f"{dir_head}/{fname}"
        func_sig = i['sig']
        gen_cls = type(fname,
                       (view_cls,),
                       {})
        gen_cls.post = copy_func(gen_cls.post)
        gen_cls.post.__doc__ = i['doc'] + '\n\n' + \
            rest_api_auto_doc(apidocstr, func_sig)
        globals()[fname] = gen_cls
        global generated_urls
        generated_urls.append(
            path(apidocstr, globals()[fname].as_view(), name=fname))


generate_apis(super_master._public_api, AbstractPublicJacAPIView, 'public')
generate_apis(super_master._private_api, AbstractJacAPIView, 'jac')
generate_apis(super_master._admin_api, AbstractAdminJacAPIView, 'admin')



from django.contrib.auth import get_user_model
from django.urls import reverse

from rest_framework.test import APIClient
from jaseci.utils.utils import TestCaseHelper
import jaseci.actions.live_actions as lact
from django.test import TestCase
import uuid
import base64
import os


class test_ll_wall(TestCaseHelper, TestCase):
    """Test the authorized user node API"""

    def setUp(self):
        super().setUp()
        self.user = get_user_model().objects.create_user(
            'JSCITfdfdEST_test@jaseci.com',
            'password'
        )
        self.client = APIClient()
        self.client.force_authenticate(self.user)
        ll_loc = os.path.dirname(__file__) + '/ll_wall.jac'
        ll_file = base64.b64encode(
            open(ll_loc).read().encode()).decode()
        payload = {'op': 'sentinel_register',
                   'name': 'Something', 'code': ll_file, 'encoded': True}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        payload = {'op': 'global_sentinel_set'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.user = get_user_model().objects.create_user(
            'J2SCITfdfdEST_test@jaseci.com',
            'password'
        )
        self.client = APIClient()
        self.client.force_authenticate(self.user)
        self.master = self.user.get_master()
        payload = {'op': 'sentinel_active_global'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'graph_create'}
        self.snt = self.master._h.get_obj(
            self.master.jid, uuid.UUID(res.data['sentinel']['jid']))
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.gph = self.master._h.get_obj(
            self.master.jid, uuid.UUID(res.data['jid']))
        lact.load_local_actions('jaseci_serv/jac_api/tests/infer.py')

    def tearDown(self):
        super().tearDown()

    def run_walker(self, w_name, ctx, prime=None):
        """Helper to make calls to execute walkers"""
        if(not prime):
            payload = {'snt': self.snt.id.urn, 'name': w_name,
                       'nd': self.gph.id.urn, 'ctx': ctx}
        else:
            payload = {'snt': self.snt.id.urn, 'name': w_name,
                       'nd': prime, 'ctx': ctx}
        res = self.client.post(
            reverse('jac_api:walker_run'), payload, format='json')
        return res.data

    def graph_node_set(self, nd_id, ctx):
        """Helper to set node context"""
        payload = {'snt': self.snt.id.urn, 'nd': nd_id, 'ctx': ctx}
        res = self.client.post(
            reverse('jac_api:graph_node_set'), payload, format='json')
        return res.data

    def test_ll_wall_get_gen_day(self):
        """Test get_gen_day walker response time after cerify day"""
        num_workettes = 112

        # generate random day workettes
        self.run_walker('gen_day_workettes', {
                        "date": "2021-07-12", "num_workettes": num_workettes})

        data = self.run_walker('get_latest_day', {'show_report': 1})['report']

        day_id = data[0][1]['jid']
        day_date = data[0][1]['context']['day']
        day_note = data[0][1]['context']['note']

        data = self.run_walker('get_workettes_deep', {
                               'show_report': 1}, prime=day_id)['report']
        self.assertEqual(len(data[0]), num_workettes)

        # certify day, should return day highlights
        data = self.run_walker('set_day_highlight', {"highlight_items": [
            {"id": data[0][0], "type": "Most proud accomplishment",
                "color": "#464ff6", "icon": "0x1F3C6"},
            {"id": data[0][1], "type": "Made You Happiest",
             "color": "#6e30dd", "icon": "0x1F604"},
            {"id": data[0][2], "type": "Required the Most Work",
             "color": "#b926df", "icon": "0x1F4AA"}
        ]}, prime=day_id)['report']
        self.assertEqual(data[0][0][0][0][0]['jid'], day_id)
        self.assertEqual(data[0][0][0][0][1], day_date)
        self.assertEqual(data[0][0][0][1], day_note)

        # data[0][0][0][2] is the highlight items
        self.assertEqual(len(data[0][0][0][2]), 3)

        data = self.run_walker('get_gen_day', {"date": "2021-07-13"})



from rest_framework.views import APIView
from knox.auth import TokenAuthentication
from rest_framework.permissions import IsAdminUser, IsAuthenticated, AllowAny
from rest_framework.response import Response
from jaseci.utils.utils import logger
from jaseci.element.element import element
from jaseci_serv.base.orm_hook import orm_hook
from jaseci_serv.base.models import JaseciObject, GlobalVars
from jaseci_serv.base.models import master as core_master
from time import time


class JResponse(Response):
    def __init__(self, master, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.master = master
        for i in self.master._h.save_obj_list:
            self.master._h.commit_obj_to_redis(i)
        self.master._h.skip_redis_update = True

    def close(self):
        super(JResponse, self).close()
        # Commit db changes after response to user
        self.master._h.commit()


class AbstractJacAPIView(APIView):
    """
    The builder set of Jaseci APIs
    """
    authentication_classes = (TokenAuthentication,)
    permission_classes = (IsAuthenticated,)

    def post(self, request):
        """
        General post function that parses api signature to load parms
        SuperSmart Post - can read signatures of master and process
        bodies accordingly
        """
        self.proc_request(request)

        api_result = self.caller.general_interface_to_api(
            self.cmd, type(self).__name__)
        self.log_request_stats()
        return self.issue_response(api_result)

    def log_request_stats(self):
        """Api call preamble"""
        TY = '\033[33m'
        TG = '\033[32m'
        EC = '\033[m'  # noqa
        tot_time = time()-self.start_time
        save_count = 0
        if(isinstance(self.caller, element)):
            save_count = len(self.caller._h.save_obj_list)
        logger.info(str(
            f'API call to {TG}{type(self).__name__}{EC}'
            f' completed in {TY}{tot_time:.3f} seconds{EC}'
            f' saving {TY}{save_count}{EC} objects.'))

    def proc_request(self, request):
        """Parse request to field set"""
        pl_peek = str(dict(request.data))[:256]
        logger.info(str(
            f'Incoming call to {type(self).__name__} with {pl_peek}'))
        self.start_time = time()
        self.cmd = request.data
        self.set_caller(request)
        self.res = "Not valid interaction!"

    def set_caller(self, request):
        """Assigns the calling api interface obj"""
        self.caller = request.user.get_master()

    def issue_response(self, api_result):
        """Issue response from call"""
        # self.caller._h.commit()
        # return Response(api_result)
        # for i in self.caller._h.save_obj_list:
        #     self.caller._h.commit_obj_to_redis(i)
        return JResponse(self.caller, api_result)


class AbstractAdminJacAPIView(AbstractJacAPIView):
    """
    The abstract base for Jaseci Admin APIs
    """
    permission_classes = (IsAuthenticated, IsAdminUser)


class AbstractPublicJacAPIView(AbstractJacAPIView):
    """
    The abstract base for Jaseci Admin APIs
    """
    permission_classes = (AllowAny,)

    def post(self, request):
        """
        Public post function that parses api signature to load parms
        SuperSmart Post - can read signatures of master and process
        bodies accordingly
        """
        self.proc_request(request)

        api_result = self.caller.public_interface_to_api(
            self.cmd, type(self).__name__)
        self.log_request_stats()
        return self.issue_response(api_result)

    def set_caller(self, request):
        """Assigns the calling api interface obj"""
        self.caller = core_master(h=orm_hook(
            objects=JaseciObject.objects,
            globs=GlobalVars.objects
        ), persist=False)

    def issue_response(self, api_result):
        """Issue response from call"""
        # If committer set, results should be saved back
        if(self.caller._pub_committer):
            return JResponse(self.caller._pub_committer, api_result)
        else:
            return Response(api_result)



from django.contrib.auth import get_user_model, authenticate
from django.utils.translation import ugettext_lazy as _

from rest_framework import serializers
import base64
from django.urls import reverse
from django.dispatch import receiver
from django_rest_passwordreset.signals import reset_password_token_created
from jaseci_serv.base.mail import email_config


def send_activation_email(request, email):
    """Construct activation email body"""
    code = base64.b64encode(email.encode()).decode()
    link = request.build_absolute_uri(
        reverse('user_api:activate', kwargs={'code': code}))
    jsmail = email_config()
    jsmail.send_activation_email(email, code, link)


@receiver(reset_password_token_created)
def password_reset_token_created(sender, instance, reset_password_token,
                                 *args, **kwargs):
    jsmail = email_config()
    jsmail.send_reset_email(
        reset_password_token.user.email, reset_password_token.key)


class UserSerializer(serializers.ModelSerializer):
    """Serializer for users object"""

    class Meta:
        model = get_user_model()
        fields = ('id', 'email', 'password', 'name', 'is_activated')
        extra_kwargs = {'password': {'write_only': True, 'min_length': 8}}

    def create(self, validated_data):
        """Create a new user with encrypted password and return it"""
        return get_user_model().objects.create_user(**validated_data)

    def update(self, instance, validated_data):
        """Update user, setting the password if needed"""
        password = validated_data.pop('password', None)
        user = super().update(instance, validated_data)

        if password:
            user.set_password(password)
            user.save()

        return user


class SuperUserSerializer(UserSerializer):
    """Serializer for users object"""

    def create(self, validated_data):
        """Create a new user with encrypted password and return it"""
        return get_user_model().objects.create_superuser(**validated_data)


class AuthTokenSerializer(serializers.Serializer):
    """Serializer for the user authentication object"""
    email = serializers.CharField()
    password = serializers.CharField(
        style={'input_type': 'password'},
        trim_whitespace=False
    )

    def validate(self, attrs):
        """
        Validate and authenticate the user
        """
        email = attrs.get('email').lower()
        password = attrs.get('password')

        user = authenticate(
            request=self.context.get('request'),
            username=email,
            password=password
        )
        if not user:
            msg = _('Unable to authenticate with provided credentials')
            raise serializers.ValidationError(msg, code='authorization')
        elif not user.is_activated:
            msg = _('User not activated. Resending activation email.\n' +
                    'Please check your email.')
            send_activation_email(self.context.get('request'), email)
            raise serializers.ValidationError(msg)

        attrs['user'] = user
        return attrs



"""
This module includes code related to configuring Jaseci's mail serving
"""
from jaseci_serv.base.models import lookup_global_config
from django.core import mail
from jaseci.utils.utils import logger


email_config_defaults = {
    'EMAIL_BACKEND': 'django.core.mail.backends.smtp.EmailBackend',
    'EMAIL_USE_TLS': True,
    'EMAIL_HOST': 'smtp.gmail.com',
    'EMAIL_HOST_USER': 'prod@lifelogify.com',
    'EMAIL_DEFAULT_FROM': 'hello@jaseci.com',
    'EMAIL_HOST_PASSWORD': 'pwzaijgtrupomcjy',
    'EMAIL_PORT': 587,
}

email_template_defaults = {
    'EMAIL_ACTIVATION_SUBJ': 'Please activate your account!',
    'EMAIL_ACTIVATION_BODY': "Thank you for creating an account!\n\n"
                             "Activation Code: {{code}}\n"
                             "Please click below to activate:\n{{link}}",
    'EMAIL_ACTIVATION_HTML_BODY': "Thank you for creating an account!<br><br>"
                                  "Activation Code: {{code}}<br>"
                                  "Please click below to activate:<br>"
                                  "{{link}}",
    'EMAIL_RESETPASS_SUBJ': "Password Reset for Jaseci Account",
    'EMAIL_RESETPASS_BODY': "Your Jaseci password reset token is: {{token}}",
    'EMAIL_RESETPASS_HTML_BODY': "Your Jaseci password reset"
                                 "token is: {{token}}",
}


class email_config():
    def __init__(self):
        self.setup_email_connection()  # creates self.connection
        self.setup_email_templates()

    def setup_email_connection(self):
        """Loads an email connection relative to global configs"""
        def resolve(name):
            return lookup_global_config(name=name,
                                        default=email_config_defaults[name])
        backend = resolve('EMAIL_BACKEND')
        host = resolve('EMAIL_HOST')
        port = int(resolve('EMAIL_PORT'))
        username = resolve('EMAIL_HOST_USER')
        password = resolve('EMAIL_HOST_PASSWORD')
        use_tls = bool(resolve('EMAIL_USE_TLS'))
        self.sender = resolve('EMAIL_DEFAULT_FROM')

        self.connection = mail.get_connection(
            backend=backend, host=host, port=port,
            username=username, password=password,
            use_tls=use_tls)

    def setup_email_templates(self):
        """Loads email content templates from global config"""
        def resolve(name):
            return lookup_global_config(name=name,
                                        default=email_template_defaults[name])
        self.activ_subj = resolve('EMAIL_ACTIVATION_SUBJ')
        self.activ_body = resolve('EMAIL_ACTIVATION_BODY')
        if('{{code}}' not in self.activ_body and
           '{{link}}' not in self.activ_body):
            logger.error("{{code/link}} must be present in email template")
            self.activ_body = email_template_defaults['EMAIL_ACTIVATION_BODY']
        self.activ_html = resolve('EMAIL_ACTIVATION_HTML_BODY')
        if('{{code}}' not in self.activ_html and
           '{{link}}' not in self.activ_html):
            logger.error("{{code/link}} must be present in email template")
            self.activ_body = email_template_defaults[
                'EMAIL_ACTIVATION_HTML_BODY']
        self.reset_subj = resolve('EMAIL_RESETPASS_SUBJ')
        self.reset_body = resolve('EMAIL_RESETPASS_BODY')
        if('{{token}}' not in self.reset_body):
            logger.error("{{token}} must be present in email template")
            self.reset_body = email_template_defaults['EMAIL_RESETPASS_BODY']
        self.reset_html = resolve('EMAIL_RESETPASS_HTML_BODY')
        if('{{token}}' not in self.reset_html):
            logger.error("{{token}} must be present in email template")
            self.reset_html = email_template_defaults[
                'EMAIL_RESETPASS_HTML_BODY']

    def send_activation_email(self, email, code, link):
        """Apply relevant parameters to loaded templates"""
        body = self.activ_body.replace(
            '{{code}}', code).replace('{{link}}', link)
        html = self.activ_html.replace(
            '{{code}}', code).replace('{{link}}', link)
        with self.connection as connection:
            msg = mail.EmailMultiAlternatives(
                subject=self.activ_subj,
                body=body,
                from_email=self.sender,
                to=[email],
                connection=connection,
            )
            msg.attach_alternative(html, "text/html")
            msg.send(fail_silently=False)

    def send_reset_email(self, email, token):
        """Apply relevant parameters to loaded templates"""
        body = self.reset_body.replace('{{token}}', token)
        html = self.reset_html.replace('{{token}}', token)
        with self.connection as connection:
            msg = mail.EmailMultiAlternatives(
                subject=self.reset_subj,
                body=body,
                from_email=self.sender,
                to=[email],
                connection=connection,
            )
            msg.attach_alternative(html, "text/html")
            msg.send(fail_silently=False)



import base64

from django.contrib.auth import get_user_model
from django.urls import reverse

from rest_framework.test import APIClient
from rest_framework import status

from jaseci.utils.utils import TestCaseHelper
from django.test import TestCase

import uuid
import os


class PublicJacApiTests(TestCaseHelper, TestCase):
    """Test the publicly available node API"""

    def setUp(self):
        super().setUp()
        self.client = APIClient()
        get_user_model().objects.create_user(
            'throwawayJSCITfdfdEST_test@jaseci.com',
            'password'
        )
        self.user = get_user_model().objects.create_user(
            'JSCITfdfdEST_test@jaseci.com',
            'password'
        )
        self.auth_client = APIClient()
        self.auth_client.force_authenticate(self.user)
        self.suser = get_user_model().objects.create_superuser(
            'JSCITfdfdEST_test2@jaseci.com',
            'password'
        )
        self.sauth_client = APIClient()
        self.sauth_client.force_authenticate(self.suser)

    def tearDown(self):
        super().tearDown()

    def test_login_required_jac_api(self):
        """Test that login required for retrieving nodes"""
        res = self.client.get(reverse('jac_api:graph_list'))

        self.assertEqual(res.status_code, status.HTTP_401_UNAUTHORIZED)

    def test_public_jac_apis_walker_summon_auth(self):
        """Test public API for summoning walker"""
        zsb_file = open(os.path.dirname(__file__) +
                        "/zsb.jac").read()
        payload = {'op': 'sentinel_register', 'name': 'zsb',
                   'code': zsb_file}
        res = self.auth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_get', 'mode': 'keys',
                   'wlk': 'zsb:walker:pubinit'}
        res = self.auth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        key = res.data['anyone']
        payload = {'op': 'alias_list'}
        res = self.auth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        walk = res.data['zsb:walker:pubinit']
        nd = res.data['active:graph']
        payload = {'op': 'walker_summon', 'key': key, 'wlk': walk, 'nd': nd}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data['report']), 0)
        key = 'aaaaaaa'
        payload = {'op': 'walker_summon', 'key': key, 'wlk': walk, 'nd': nd}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertFalse(res.data['success'])

    def test_serverside_sentinel_global_public_access_summon(self):
        """Test master delete operation"""
        zsb_file = open(os.path.dirname(__file__) +
                        "/zsb.jac").read()
        payload = {'op': 'sentinel_register', 'name': 'zsb',
                   'code': zsb_file}
        res = self.sauth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 2)
        payload = {'op': 'global_sentinel_set'}
        res = self.sauth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'sentinel_active_global'}
        res = self.auth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertTrue(res.data['success'])
        payload = {'op': 'graph_create'}
        res = self.auth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_spawn_create', 'name': 'pubinit'}
        res = self.auth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_get', 'mode': 'keys',
                   'wlk': 'spawned:walker:pubinit'}
        res = self.auth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        key = res.data['anyone']
        payload = {'op': 'alias_list'}
        res = self.auth_client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        walk = res.data['spawned:walker:pubinit']
        nd = res.data['active:graph']
        payload = {'op': 'walker_summon', 'key': key, 'wlk': walk, 'nd': nd}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data['report']), 0)


class PrivateJacApiTests(TestCaseHelper, TestCase):
    """Test the authorized user node API"""

    def setUp(self):
        super().setUp()
        get_user_model().objects.create_user(
            'throwawayJSCITfdfdEST_test@jaseci.com',
            'password'
        )
        self.user = get_user_model().objects.create_user(
            'JSCITfdfdEST_test@jaseci.com',
            'password'
        )
        self.suser = get_user_model().objects.create_superuser(
            'JSCITfdfdEST_test2@jaseci.com',
            'password'
        )
        self.client = APIClient()
        self.client.force_authenticate(self.user)
        self.sclient = APIClient()
        self.sclient.force_authenticate(self.suser)
        self.master = self.user.get_master()

    def tearDown(self):
        super().tearDown()

    def test_jac_api_create_graph(self):
        """Test API for creating a graph"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        gph = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        self.assertEqual(gph.name, "root")

    def test_jac_api_create_sentinel(self):
        """Test API for creating a sentinel"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.master._h.get_obj(self.master.j_master,
                               uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Untitled Sentinel'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        sent = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertEqual(sent.name, "Untitled Sentinel")

    def test_jac_api_list_graphs_sents(self):
        """Test API for creating a sentinel"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        payload = {'op': 'graph_list'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        glist = res.data
        payload = {'op': 'sentinel_register', 'name': 'SomeSent'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        payload = {'op': 'sentinel_register', 'name': 'SomeSent2'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        payload = {'op': 'sentinel_list'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        slist = res.data
        self.assertEqual(len(glist), 1)
        self.assertEqual(len(slist), 2)
        self.assertEqual(slist[1]['name'], 'SomeSent2')
        self.assertEqual(glist[0]['name'], 'root')

    def test_jac_api_delete_graph(self):
        """Test API for deleteing a graph"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        gph = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Something'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.assertIn(gph.id, gph._h.mem.keys())
        payload = {'op': 'graph_delete', 'gph': gph.id.urn}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.assertNotIn(gph.id, gph._h.mem.keys())

    def test_jac_api_get_graph_dot(self):
        """Test API for getting graph in dot str"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        gph = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'graph_get', 'mode': 'dot',
                   'gph': gph.id.urn, 'dot': True}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.assertTrue('graph root' in res.json())

    def test_jac_api_delete_sentinel(self):
        """Test API for deleting a sentinel"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        gph = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Something'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        sent = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        self.assertIn(gph.id, gph._h.mem.keys())
        self.assertIn(sent.id, gph._h.mem.keys())
        payload = {'op': 'sentinel_delete', 'snt': sent.id.urn}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.assertIn(gph.id, gph._h.mem.keys())
        self.assertNotIn(sent.id, gph._h.mem.keys())

    def test_jac_api_get_jac_code(self):
        """Test API for deleting a sentinel"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.master._h.get_obj(self.master.j_master,
                               uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Something',
                   'code': 'walker testwalker{ std.log("hello"); }', }
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        sent = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        payload = {'op': 'sentinel_get', 'mode': 'code', 'snt': sent.id.urn}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.assertIn('walker test', res.data)

    def test_jac_api_sentinel_set(self):
        """Test API for deleting a sentinel"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.master._h.get_obj(self.master.j_master,
                               uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Something'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        sent = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        payload = {'op': 'sentinel_set', 'snt': sent.id.urn,
                   'code': 'node awesome;', 'encoded': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertTrue(sent.code_ir.startswith('{"name": "start"'))

    def test_jac_api_sentinel_set_encoded(self):
        """Test API for deleting a sentinel"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.master._h.get_obj(self.master.j_master,
                               uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Something'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        sent = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        enc_str = base64.b64encode(b'node awesome;').decode()
        payload = {'op': 'sentinel_set', 'snt': sent.id.urn,
                   'code': enc_str, 'encoded': True}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertTrue(sent.code_ir.startswith('{"name": "start"'))

    def test_jac_api_compile(self):
        """Test API for compiling a sentinel"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.master._h.get_obj(self.master.j_master,
                               uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Something'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        sent = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        payload = {'op': 'sentinel_set', 'snt': sent.id.urn,
                   'code': 'walker testwalker{ std.log("hello"); }',
                   'encoded': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertTrue(sent.is_active)
        self.assertEqual(sent.walker_ids.obj_list()[0].name, 'testwalker')

    def test_jac_api_load_application(self):
        """Test API for loading an application"""
        enc_str = base64.b64encode(
            b'walker testwalker{ std.log("hello"); }').decode()
        payload = {'op': 'sentinel_register', 'name': 'test_app',
                   'code': enc_str, 'encoded': True}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        snt = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        gph = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[1]['jid']))
        self.assertEqual(snt.name, "test_app")
        self.assertEqual(gph.name, "root")
        self.assertTrue(snt.is_active)

    def test_jac_api_load_application_no_leak(self):
        """Test API for loading an application"""
        enc_str = base64.b64encode(
            b'node sample { has apple;} ' +
            b'walker testwalker{ new = spawn here --> node::sample; ' +
            b'report new; }').decode()
        payload = {'op': 'sentinel_register', 'name': 'test_app',
                   'code': enc_str, 'encoded': True}
        num_objs_a = len(self.master._h.mem.keys())
        self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        num_objs_b = len(self.master._h.mem.keys())
        self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        num_objs_c = len(self.master._h.mem.keys())
        self.assertLess(num_objs_a, num_objs_b)
        self.assertEqual(num_objs_b, num_objs_c)

    def test_jac_api_spawn(self):
        """Test API for spawning a walker"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.master._h.get_obj(self.master.j_master,
                               uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Something'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        sent = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        payload = {'op': 'sentinel_set', 'snt': sent.id.urn,
                   'code': 'walker testwalker{ std.log("hello"); }',
                   'encoded': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_spawn_create', 'snt': sent.id.urn,
                   'name': 'testwalker'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        walk = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        self.assertNotEqual(walk.id, sent.walker_ids.obj_list()[0].id)
        self.assertEqual(walk.name, sent.walker_ids.obj_list()[0].name)

    def test_jac_api_prime(self):
        """Test API for priming a walker"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        gph = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Something'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        sent = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        payload = {'op': 'sentinel_set', 'snt': sent.id.urn,
                   'code': 'walker testwalker{ std.log("hello"); }',
                   'encoded': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_spawn_create', 'snt': sent.id.urn,
                   'name': 'testwalker'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        walk = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'walker_prime', 'wlk': walk.id.urn,
                   'nd': gph.id.urn, 'ctx': {}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')

        self.assertIn(gph, walk.next_node_ids.obj_list())

    def test_jac_api_run(self):
        """Test API for running a walker"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        gph = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Something'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        sent = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        payload = {'op': 'sentinel_set', 'snt': sent.id.urn,
                   'code': 'walker testwalker{ std.log("hello"); }',
                   'encoded': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_spawn_create', 'snt': sent.id.urn,
                   'name': 'testwalker'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        walk = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'walker_prime', 'wlk': walk.id.urn,
                   'nd': gph.id.urn, 'ctx': {}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_execute', 'wlk': walk.id.urn}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.assertGreater(walk.current_step, 0)

    def test_jac_api_graph_node_set(self):
        """Test API for setting context variables of node"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        gph = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Something'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        sent = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        payload = {'op': 'sentinel_set', 'snt': sent.id.urn,
                   'code': 'node sample { has apple;} ' +
                   'walker testwalker{ new = spawn here --> node::sample; ' +
                   'report new; }',
                   'encoded': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_spawn_create', 'snt': sent.id.urn,
                   'name': 'testwalker'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        walk = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'walker_prime', 'wlk': walk.id.urn,
                   'nd': gph.id.urn, 'ctx': {}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_execute', 'wlk': walk.id.urn}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        nid = res.data['report'][0]['jid']
        payload = {'op': 'graph_node_set', 'snt': sent.id.urn, 'nd': nid,
                   'ctx': {'apple': 'TEST'}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertGreater(walk.current_step, 0)
        self.assertEqual(res.data['context']['apple'], 'TEST')

    def test_deep_serialize_report(self):
        """Test API for running a walker"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        gph = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Something'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        sent = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        payload = {
            'op': 'sentinel_set', 'snt': sent.id.urn,
            'code': 'walker testwalker{ report '
                    '[[[[[here, here], here], here]]]; }',
            'encoded': False
        }
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_spawn_create', 'snt': sent.id.urn,
                   'name': 'testwalker'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        walk = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'walker_prime', 'wlk': walk.id.urn,
                   'nd': gph.id.urn, 'ctx': {}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_execute', 'wlk': walk.id.urn}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.assertGreater(walk.current_step, 0)

    def test_create_new_fields_auto_on_old_data(self):
        """Test API for running a walker"""
        payload = {'op': 'graph_create'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        gph = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'sentinel_register', 'name': 'Something'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        sent = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data[0]['jid']))
        payload = {'op': 'sentinel_set', 'snt': sent.id.urn,
                   'code':
                   'node a { has b; } walker testwalker' +
                   '{ r = spawn here --> node::a; r.b = 6; }',
                   'encoded': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_spawn_create', 'snt': sent.id.urn,
                   'name': 'testwalker'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        walk = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'walker_prime', 'wlk': walk.id.urn,
                   'nd': gph.id.urn, 'ctx': {}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_execute', 'wlk': walk.id.urn}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.assertGreater(walk.current_step, 0)

        payload = {'op': 'sentinel_set', 'snt': sent.id.urn,
                   'code':
                   'node a { has b, c; } walker testwalker' +
                   '{ with entry {take -->;} ' +
                   'a { here.c=7; std.log(here.c); }}',
                   'encoded': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_spawn_create', 'snt': sent.id.urn,
                   'name': 'testwalker'}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        walk = self.master._h.get_obj(
            self.master.j_master, uuid.UUID(res.data['jid']))
        payload = {'op': 'walker_prime', 'wlk': walk.id.urn,
                   'nd': gph.id.urn, 'ctx': {}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_execute', 'wlk': walk.id.urn}
        res = self.client.post(reverse(f'jac_api:{payload["op"]}'), payload)
        self.assertEqual(gph.outbound_nodes()[0].context['c'], 7)

    def test_master_create_linked_to_django_users(self):
        """Test master create operation"""
        payload = {'op': 'master_create', 'name': 'yo@gmail.com',
                   'other_fields': {'password': 'yoyoyoyoyoyo', 'name': '',
                                    'is_activated': True}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertIn('j_type', res.data)
        self.assertEqual(res.data['j_type'], 'master')

    def test_master_create_linked_error_out(self):
        """Test master create operation"""
        payload = {'op': 'master_create', 'name': 'yo@gmail.com',
                   'other_fields': {'password': 'yoyoyoyoyoyo', 'name': '',
                                    'is_activated': True}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'master_create', 'name': 'yo@gmail.com',
                   'other_fields': {'password': 'yoyo', 'name': '',
                                    'is_activated': True}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertIn('errors', res.data)
        self.assertIn('email', res.data['errors'])
        self.assertIn('password', res.data['errors'])

    def test_master_create_linked_cant_override(self):
        """Test master create operation"""
        payload = {'op': 'master_create', 'name': 'yo2@gmail.com',
                   'other_fields': {'password': 'yoyoyoyoyo', 'name': '',
                                    'is_activated': True, 'is_admin': True,
                                    'is_staff': True, 'is_superuser': True}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.data['j_type'], 'master')

    def test_master_create_linked_super_limited(self):
        """Test master create operation"""
        payload = {'op': 'master_createsuper', 'name': 'yo3@gmail.com',
                   'other_fields': {'password': 'yoyoyoyoyo', 'name': '',
                                    'is_activated': True, 'is_admin': True,
                                    'is_staff': True, 'is_superuser': True}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.status_code, 403)

    def test_master_create_linked_super_master_create(self):
        """Test master create operation"""
        payload = {'op': 'master_createsuper', 'name': 'yo3@gmail.com',
                   'other_fields': {'password': 'yoyoyoyoyo', 'name': '',
                                    'is_activated': True, 'is_admin': True,
                                    'is_staff': True, 'is_superuser': True}}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertIn('j_type', res.data)
        self.assertEqual(res.data['j_type'], 'super_master')

    def test_master_create_linked_to_django_users_login(self):
        """Test master create operation"""
        payload = {'op': 'master_create', 'name': 'yo@gmail.com',
                   'other_fields': {'password': 'yoyoyoyoyoyo', 'name': '',
                                    'is_activated': True}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        login_client = APIClient()
        payload = {'email': 'yo@gmail.com', 'password': 'yoyoyoyoyoyo'}
        res = login_client.post(reverse('user_api:token'), payload)
        self.assertIn('token', res.data)

    def test_master_create_linked_survives_ORM(self):
        """Test master create operation"""
        self.user.get_master()._h.clear_mem_cache()
        payload = {'op': 'master_create', 'name': 'yo@gmail.com',
                   'other_fields': {'password': 'yoyoyoyoyoyo', 'name': '',
                                    'is_activated': True}}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        login_client = APIClient()
        payload = {'email': 'yo@gmail.com', 'password': 'yoyoyoyoyoyo'}
        res = login_client.post(reverse('user_api:token'), payload)
        self.assertIn('token', res.data)

    def test_master_create_super_inherets_from_django(self):
        """Test master create operation"""
        payload = {'op': 'master_create', 'name': 'yo@gmail.com',
                   'other_fields': {'password': 'yoyoyoyoyoyo', 'name': '',
                                    'is_activated': True}}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        login_client = APIClient()
        payload = {'email': 'yo@gmail.com', 'password': 'yoyoyoyoyoyo'}
        res = login_client.post(reverse('user_api:token'), payload)
        self.assertIn('token', res.data)

    def test_master_delete_linked_to_django(self):
        """Test master delete operation"""
        payload = {'op': 'master_create', 'name': 'yo@gmail.com',
                   'other_fields': {'password': 'yoyoyoyoyoyo', 'name': '',
                                    'is_activated': True}}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        login_client = APIClient()
        payload = {'email': 'yo@gmail.com', 'password': 'yoyoyoyoyoyo'}
        res = login_client.post(reverse('user_api:token'), payload)
        self.assertIn('token', res.data)
        payload = {'op': 'master_delete', 'name': 'yo@gmail.com'}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        login_client = APIClient()
        payload = {'email': 'yo@gmail.com', 'password': 'yoyoyoyoyoyo'}
        res = login_client.post(reverse('user_api:token'), payload)
        self.assertNotIn('token', res.data)

    def test_serverside_sentinel_register_global(self):
        """Test master delete operation"""
        zsb_file = open(os.path.dirname(__file__) +
                        "/zsb.jac").read()
        payload = {'op': 'sentinel_register', 'name': 'zsb',
                   'code': zsb_file}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 2)
        payload = {'op': 'global_sentinel_set'}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'sentinel_active_global'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertTrue(res.data['success'])
        payload = {'op': 'sentinel_active_get'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertIn('jid', res.data)
        payload = {'op': 'graph_list'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 0)
        payload = {'op': 'graph_create'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'graph_get'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 1)
        payload = {'op': 'walker_run', 'name': 'init'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'graph_get'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 3)

    def test_serverside_sentinel_unregister_global(self):
        """Test master delete operation"""
        zsb_file = open(os.path.dirname(__file__) +
                        "/zsb.jac").read()
        payload = {'op': 'sentinel_register', 'name': 'zsb',
                   'code': zsb_file}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 2)
        payload = {'op': 'global_sentinel_set'}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'sentinel_active_global'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertTrue(res.data['success'])
        payload = {'op': 'graph_create'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_run', 'name': 'init'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'graph_get'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 3)
        payload = {'op': 'graph_create'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'graph_get'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 1)
        payload = {'op': 'global_sentinel_unset'}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.user._h.clear_mem_cache()
        payload = {'op': 'walker_run', 'name': 'init'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'graph_get'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 1)

    def test_serverside_sentinel_register_pull(self):
        """Test master delete operation"""
        zsb_file = open(os.path.dirname(__file__) +
                        "/zsb.jac").read()
        payload = {'op': 'sentinel_register', 'name': 'zsb',
                   'code': zsb_file}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 2)
        payload = {'op': 'global_sentinel_set'}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'sentinel_pull'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertTrue(res.data['success'])
        payload = {'op': 'sentinel_active_get'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertIn('jid', res.data)
        payload = {'op': 'graph_list'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 0)
        payload = {'op': 'graph_create'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'graph_get'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 1)
        payload = {'op': 'walker_run', 'name': 'init'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'graph_get'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(len(res.data), 3)

    def test_check_json_global_dict(self):
        """Test set get global objects (as json)"""
        from jaseci.tests.jac_test_code import set_get_global_dict
        payload = {'op': 'sentinel_register', 'name': 'zsb',
                   'code': set_get_global_dict}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_run', 'name': 'setter'}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_run', 'name': 'getter'}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.data['report'][0]['max_bot_count'], 10)

    def test_check_sentinel_set(self):
        """Test that sentinel set works serverside"""
        import jaseci.tests.jac_test_code as jtc
        payload = {'op': 'sentinel_register', 'name': 'zsb',
                   'auto_run': '', 'code': jtc.version_label}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'sentinel_set', 'code': jtc.set_get_global_dict}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_run', 'name': 'setter'}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'walker_run', 'name': 'getter'}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.data['report'][0]['max_bot_count'], 10)

    def test_action_loads_remote_config_save(self):
        """Test that sentinel set works serverside"""
        import jaseci.actions.live_actions as lact
        import json
        if (not lact.load_remote_actions('http://js-use-enc')):
            self.skipTest("external resource not available")
        payload = {'op': 'config_delete', 'name': 'ACTION_SETS'}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'actions_load_remote', 'url': 'http://js-use-enc'}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.data, {'success': True})
        payload = {'op': 'config_get', 'name': 'ACTION_SETS'}
        res = self.sclient.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(
            res.data,
            json.dumps({"local": [], "remote": ["http://js-use-enc"]}))



from django.conf import settings
from django.core.management.base import BaseCommand
import subprocess


class Command(BaseCommand):
    """
    Drop postgres database
    https://www.postgresql.org/docs/current/app-dropdb.html
    """

    def add_arguments(self, parser):
        super(Command, self).add_arguments(parser)
        parser.add_argument('alias', nargs='?', default="default")

    def handle(self, *args, **options):
        alias = options.get('alias')

        db_settings = settings.DATABASES[alias]
        args = ['dropdb']
        if 'USER' in db_settings:
            args += ["-U", db_settings['USER']]
        if 'HOST' in db_settings:
            args += ["-h", db_settings['HOST']]
        if 'PORT' in db_settings:
            args += ["-p", str(db_settings['PORT'])]
        args.append(db_settings['NAME'])
        subprocess.check_call(args)



from jaseci.utils.mem_hook import mem_hook
from jaseci.actor.sentinel import sentinel
from jaseci.graph.graph import graph

from jaseci.utils.utils import TestCaseHelper
from django.test import TestCase
import jaseci.tests.jac_test_code as jtc
import jaseci.actions.live_actions as lact


class jac_tests(TestCaseHelper, TestCase):
    """Unit tests for Jac language"""

    def setUp(self):
        super().setUp()

    def tearDown(self):
        super().tearDown()

    def test_basic_USE_calls_from_jac(self):
        """Test the execution of a basic walker building graph"""
        if (not lact.load_remote_actions('http://js-use-qa')):
            self.skipTest("external resource not available")
        gph = graph(m_id='anon', h=mem_hook())
        sent = sentinel(m_id=gph._m_id, h=gph._h)
        sent.register_code(jtc.prog1)
        test_node = sent.arch_ids.get_obj_by_name('life', kind='node').run()
        test_walker = \
            sent.walker_ids.get_obj_by_name('use_test')
        test_walker.prime(test_node)
        report = test_walker.run()
        test_walker.save()
        test_walker._h.commit()
        self.assertEqual(len(report['report'][0]), 2)
        self.assertEqual(len(report['report'][0][1]), 2)

    def test_basic_USE_single_string_calls_from_jac(self):
        """Test the execution of a basic walker building graph"""
        if (not lact.load_remote_actions('http://js-use-qa')):
            self.skipTest("external resource not available")
        gph = graph(m_id='anon', h=mem_hook())
        sent = sentinel(m_id=gph._m_id, h=gph._h)
        sent.register_code(jtc.prog1)
        test_node = sent.arch_ids.get_obj_by_name('life', kind='node').run()
        test_walker = \
            sent.walker_ids.get_obj_by_name('use_test_single')
        test_walker.prime(test_node)
        report = test_walker.run()
        test_walker.save()
        test_walker._h.commit()
        self.assertEqual(len(report['report'][0]), 1)
        self.assertEqual(len(report['report'][0][0]), 1)

    def test_USE_qa_with_ctx(self):
        """Test the execution of a basic walker building graph"""
        if (not lact.load_remote_actions('http://js-use-qa')):
            self.skipTest("external resource not available")
        gph = graph(m_id='anon', h=mem_hook())
        sent = sentinel(m_id=gph._m_id, h=gph._h)
        sent.register_code(jtc.prog1)
        test_node = sent.arch_ids.get_obj_by_name('life', kind='node').run()
        test_walker = \
            sent.walker_ids.get_obj_by_name('use_test_with_ctx')
        test_walker.prime(test_node)
        report = test_walker.run()
        test_walker.save()
        test_walker._h.commit()
        self.assertEqual(len(report['report'][0]), 1)
        self.assertEqual(len(report['report'][0][0]), 1)

    def test_USE_qa_with_ctx_clean(self):
        """Test the execution of a basic walker building graph"""
        if (not lact.load_remote_actions('http://js-use-qa')):
            self.skipTest("external resource not available")
        gph = graph(m_id='anon', h=mem_hook())
        sent = sentinel(m_id=gph._m_id, h=gph._h)
        sent.register_code(jtc.prog1)
        test_node = sent.arch_ids.get_obj_by_name('life', kind='node').run()
        test_walker = \
            sent.walker_ids.get_obj_by_name('use_test_with_ctx2')
        test_walker.prime(test_node)
        report = test_walker.run()
        test_walker.save()
        test_walker._h.commit()
        self.assertEqual(len(report['report']), 8)
        self.assertEqual(len(report['report'][0]), 1)
        self.assertEqual(len(report['report'][0][0]), 1)



from setuptools import setup, find_packages
setup(
    name='jaseci_serv',
    version='1.3.2.2',
    packages=find_packages(include=['jaseci_serv', 'jaseci_serv.*']),
    install_requires=[
        'jaseci',
        'Django>=3.1.0,<3.2.0',
        'djangorestframework>=3.11.0,<3.12.0',
        'django-rest-knox>=4.1.0,<4.2.0',
        'django-rest-passwordreset>=1.1.0,<1.2.0',
        'drf-yasg>=1.20.0,<1.21.0',
        'markdown>=3.2.1,<3.3.0',
        'psycopg2-binary>=2,<3',
        'sphinx>=2.4.3,<2.5.0',
        'django-cors-headers',
        'tblib'
    ],
    package_data={"": ["*.jac"], },
    entry_points={
        'console_scripts': [
            'jsserv = jaseci_serv.manage:main'
        ]
    })



from django.contrib.auth import get_user_model
from django.urls import reverse

from rest_framework.test import APIClient
from rest_framework import status

from jaseci.utils.utils import TestCaseHelper
from django.test import TestCase


class PrivateJacAdminApiTests(TestCaseHelper, TestCase):
    """Test the authorized user node API"""

    def setUp(self):
        super().setUp()
        # First user is always super,
        self.user = get_user_model().objects.create_user(
            'throwawayJSCITfdfdEST_test@jaseci.com',
            'password'
        )
        self.nonadmin = get_user_model().objects.create_user(
            'JSCITfdfdEST_test@jaseci.com',
            'password'
        )
        self.client = APIClient()
        self.client.force_authenticate(self.user)
        self.notadminc = APIClient()
        self.notadminc.force_authenticate(self.nonadmin)
        self.master = self.user.get_master()

    def tearDown(self):
        super().tearDown()

    def test_jac_api_config_index_has_core(self):
        payload = {'op': 'config_index'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertGreater(len(res.data), 2)
        self.assertIn('ACTION_SETS', res.data)

    def test_jac_api_create_config(self):
        """Test API for creating a config"""
        payload = {'op': 'config_set', 'name': 'EMAIL_HOST_USER',
                   'value': 'val1', 'do_check': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        payload = {'op': 'config_get', 'name': 'EMAIL_HOST_USER'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertEqual(res.data, "val1")

    def test_jac_api_create_config_needs_force(self):
        """Test API for creating a config"""
        payload = {'op': 'config_delete', 'name': 'TEST'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        payload = {'op': 'config_set',
                   'name': 'TEST', 'value': 'val1'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        payload = {'op': 'config_get', 'name': 'TEST'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertNotEqual(res.data, "val1")

    def test_jac_api_create_config_nonadmin_fails(self):
        """Test API for creating a config"""
        payload = {'op': 'config_set',
                   'name': 'EMAIL_HOST_USER', 'value': 'val1'}
        res = self.notadminc.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.status_code, status.HTTP_403_FORBIDDEN)

    def test_jac_api_create_config_list(self):
        """Test API for creating a config"""
        payload = {'op': 'config_set', 'name': 'EMAIL_HOST_USER',
                   'value': 'val1', 'do_check': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        payload = {'op': 'config_set', 'name': 'EMAIL_HOST_PASSWORD',
                   'value': 'val2', 'do_check': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        payload = {'op': 'config_set', 'name': 'EMAIL_DEFAULT_FROM',
                   'value': 'val3', 'do_check': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        payload = {'op': 'config_set', 'name': 'EMAIL_BACKEND',
                   'value': 'val4', 'do_check': False}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        payload = {'op': 'config_list'}
        res = self.client.post(
            reverse(f'jac_api:{payload["op"]}'), payload, format='json')
        self.assertEqual(res.status_code, status.HTTP_200_OK)
        self.assertEqual(len(res.data), 4)
        self.assertIn('EMAIL_DEFAULT_FROM', res.data)
